#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include "RemoteDebug.h"        //https://github.com/JoaoLopesF/RemoteDebug
RemoteDebug Debug;
#include <PubSubClient.h>
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_Client.h"

const int gasPin = A0; //GAS sensor output pin to Arduino analog A0 pin
int gasValue = 0;
unsigned long previousMillis = 0;
const long interval = 15000;

WiFiClient GASsensor;
PubSubClient client(GASsensor);

// Time
uint32_t mLastTime = 0;
uint32_t mTimeSeconds = 0;
uint32_t LastTime = 0;
uint32_t TimeSeconds = 0;
#define HOST_NAME "GASsensor"
float readSensirion(unsigned int);
const char* ssid = "NomeDellaTuaReteWireless";
const char* password = "PasswordDellaTuaReteWireless";
const char* mqtt_server = "192.168.1.123";   //<-- IP del tuo RaspberryPi
const char* clientID = "GASsensor";
const char* outTopic = "SmokeSensor";
const char* questionMSG = "???";
const char* inTopic = "get/SmokeSensor";
char msg[50];
//********************************************************************************************** CALLBACK
void callback(char* topic, byte* payload, unsigned int length) {
  payload[length] = '\0'; // Null terminator used to terminate the char array
  String message = (char*)payload;

  if (strcmp (topic, inTopic) == 0) {              //topic "get/SmokeSensor"
    if (message == questionMSG) {
      dtostrf(gasValue , 0, 0, msg);
      client.publish(outTopic, msg);                   //publish SmokeSensor value
    }
  }
  Debug.println();
}
//********************************************************************************************* RECONNECT
void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Debug.println("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect(clientID)) {
      Debug.println("connected");
      // Once connected, publish an announcement...
      client.publish("Hello", clientID);
      // ... and subscribe to topic
      client.subscribe(inTopic);
    } else {
      Debug.print("failed, rc=");
      Debug.print(client.state());
      Debug.println(" try again in 5 second");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}
//************************************************************************************************* SETUP
void setup() {
  Serial.begin(115200);
  Debug.begin(HOST_NAME);
  Debug.setSerialEnabled(true);
  Debug.setResetCmdEnabled(true); // Enable the reset command
  ConnectWifi();
  while (WiFi.waitForConnectResult() != WL_CONNECTED) {
    delay (2000);
    ConnectWifi();
  }
  OTA();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
  getgas();
}
//************************************************************************************************** LOOP
void loop() {
  if (WiFi.waitForConnectResult() != WL_CONNECTED)
    ConnectWifi();
  if (!client.connected())
    reconnect();
  ArduinoOTA.handle();
  Debug.handle();

  if ((millis() - mLastTime) >= 1000) {
    mLastTime = millis();
    mTimeSeconds++;
    if (mTimeSeconds % 15 == 0) { // Each 5 seconds
      rdebugVln("* This is a message of debug level VERBOSE");
      rdebugDln("* This is a message of debug level DEBUG");
      rdebugIln("* This is a message of debug level INFO");
      rdebugWln("* This is a message of debug level WARNING");
      rdebugEln("* This is a message of debug level ERROR");
    }
    if (mTimeSeconds % 5 == 0) {
      getgas();
      dtostrf(gasValue , 0, 0, msg);
      client.publish("SmokeSensor", msg);                   //publish SmokeSensor
    }
  }
  yield();
  client.loop();
}
//****************************************************************************************** CONNECT WIFI
bool ConnectWifi() {
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);
  WiFi.begin(ssid, password);
}
//*************************************************************************************** GET GAS
void  getgas() {
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis;
    gasValue = (analogRead(gasPin));
  }
}
//*************************************************************************************** OTA
void OTA() {
  ArduinoOTA.setPort(8266);
  ArduinoOTA.setHostname(HOST_NAME);
  //ArduinoOTA.setPassword((const char *)"123");
  ArduinoOTA.onStart([]() {
    Debug.println("Start");
  });
  ArduinoOTA.onEnd([]() {
    Debug.println("\nEnd");
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
    Debug.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
    Debug.printf("Error[%u]: ", error);
    if (error == OTA_AUTH_ERROR) Debug.println("Auth Failed");
    else if (error == OTA_BEGIN_ERROR) Debug.println("Begin Failed");
    else if (error == OTA_CONNECT_ERROR) Debug.println("Connect Failed");
    else if (error == OTA_RECEIVE_ERROR) Debug.println("Receive Failed");
    else if (error == OTA_END_ERROR) Debug.println("End Failed");
  });
  ArduinoOTA.begin();
}
